-- Data export script for Lock On version 1.2.
-- Copyright (C) 2006, Eagle Dynamics.
-- See http://www.lua.org for Lua script system info 
-- We recommend to use the LuaSocket addon (http://www.tecgraf.puc-rio.br/luasocket) 
-- to use standard network protocols in Lua scripts.
-- LuaSocket 2.0 files (*.dll and *.lua) are supplied in the Scripts/LuaSocket folder
-- and in the installation folder of the Lock On version 1.2. 

-- Please, set EnableExportScript = true in the Config/Export/Config.lua file
-- to activate this script!

-- Expand the functionality of following functions for your external application needs.
-- Look into ./Temp/Error.log for this script errors, please.

-- Uncomment if using Vector class from the Config/Export/Vector.lua file 
--[[	
LUA_PATH = "?;?.lua;./Config/Export/?.lua"
require 'Vector'
-- See the Config/Export/Vector.lua file for Vector class details, please.
--]]

-- Indicator Arguments to poll for changes
gIndicators = {[175]="",[315]="",[316]="",[317]="",[318]="",[319]="",[320]="",[321]="",[322]="",[323]="",[324]="",[325]="",[326]="",
               [330]="",[331]="",[332]="",[333]="",[334]="",[335]="",[336]="",[387]="",[522]="",[560]="",[539]="",[151]="",[153]="",
               [154]="",[156]="",[541]="",[542]="",[130]="",[36]="",[21]="",[22]="",[23]="",[50]="",[16]="",[17]="",[18]="",[19]="",
               [20]="",[150]="",[159]="",[328]="",[329]="",[147]="",[228]="",[296]="",[297]="",[146]="",[388]="",[389]="",[390]="",
               [391]="",[392]="",[393]="",[394]="",[395]="",[400]="",[396]="",[435]="",[403]="",[399]="",[398]="",[56]="",[433]="",
               [434]="",[436]="",[431]="",[432]="",[438]="",[440]="",[437]="",[439]="",[441]="",[301]="",[404]="",[409]="",[25]="",
               [28]="",[26]="",[27]="",[31]="",[32]="",[33]="",[34]="",[78]="",[79]="",[80]="",[81]="",[82]="",[83]="",[84]="",
               [85]="",[86]="",[47]="",[48]="",[44]="",[37]=""}
             
-- Last sent values for weapons status
gLastWeapons = {[615]="",[2000]="",[2001]="",[2002]=""}

-- Last EKRAN value
gLastEKRAN = ""

-- Lookup tables for weapons store type display
gStationTypes = {["9A4172"] = "NC", ["S-8KOM"] = "HP", ["S-13"] = "HP", ["UPK-23-250"] = "NN", ["AO-2.5RT"] = "A6", ["PTAB-2.5KO"] = "A6",
				 ["FAB-250"] = "A6", ["FAB-500"] = "A6" }

-- Simulation id
gSimID = os.time()

-- Connection status
gConnected = false
gLastAck = 0
gAckExpected = 0
gAckAttempts = 0

-- Helper Functions
function StrSplit(str, delim, maxNb)
    -- Eliminate bad cases...
    if string.find(str, delim) == nil then
        return { str }
    end
    if maxNb == nil or maxNb < 1 then
        maxNb = 0    -- No limit
    end
    local result = {}
    local pat = "(.-)" .. delim .. "()"
    local nb = 0
    local lastPos
    for part, pos in string.gfind(str, pat) do
        nb = nb + 1
        result[nb] = part
        lastPos = pos
        if nb == maxNb then break end
    end
    -- Handle the last field
    if nb ~= maxNb then
        result[nb + 1] = string.sub(str, lastPos)
    end
    return result
end

function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

-- Status Gathering Functions
function ProcessIndicators()
    local lArgument , lLastValue , lArgumentValue
    local lSendString = ""
    local lDevice = GetDevice(0)
    lDevice:update_arguments()
    
    local lTrigger = lDevice:get_argument_value(615);
    if lTrigger == 0 then
    	gLastWeapons[615] = 1
    end
    if lTrigger == -1 then
    	gLastWeapons[615] = 0
    end
    
	for lArgument, lLastValue in pairs(gIndicators) do 
		lArgumentValue = string.format("%.1f",math.abs(lDevice:get_argument_value(lArgument)))
		if lLastValue ~= lArgumentValue then
			lSendString = lSendString..lArgument.."="..lArgumentValue..":"
			gIndicators[lArgument] = lArgumentValue
		end
	end

    return lSendString
end

function ProcessWeaponSystems()
	local lSendString = ""
	local lWeaponSystem = GetDevice(12)
	local lCannonAmmoCount = " "
	local lStationNumbers = lWeaponSystem:get_selected_weapon_stations()
	local lStationCount = " "
	local lStationType = " "
	
	if gIndicators[433] == "1.0" then
		lCannonAmmoCount = string.format("%02d",string.match(lWeaponSystem:get_selected_gun_ammo_count() / 10,"(%d+)"))
	
		if #lStationNumbers ~= 0 and gLastWeapons[615] == 0 then
			lStationCount = 0
			for i=1,#lStationNumbers do
				lStationCount = lStationCount + lWeaponSystem:get_weapon_count_on_station(lStationNumbers[i])
			end
			
			lStationCount = string.format("%02d", lStationCount);
			
			lStationType = gStationTypes[lWeaponSystem:get_weapon_type_on_station(lStationNumbers[1])]
			if lStationType == nil then
				lStationType = " "
			end
		end
	end
	
	if (gLastWeapons[2000] ~= lStationType) then
		lSendString = lSendString.."2000="..lStationType..":"
		gLastWeapons[2000] = lStationType
	end

	if (gLastWeapons[2001] ~= lStationCount) then
		lSendString = lSendString.."2001="..lStationCount..":"
		gLastWeapons[2001] = lStationCount
	end
	
	if (gLastWeapons[2002] ~= lCannonAmmoCount) then
		lSendString = lSendString.."2002="..lCannonAmmoCount..":"
		gLastWeapons[2002] = lCannonAmmoCount
	end
	
	return lSendString
end

function ProcessEKRAN()
	local lEKRAN = GetDevice(10)
	local lText = lEKRAN:get_actual_text_frame()
	local lSendString = ""
		
	if (gLastEKRAN ~= lText) then
		if (lText == "") then 
			lSendString = "2003= :2004= :2005= :2006= :"
		else
			lSendString = "2003="..string.sub(lText,1,10)..":"
			lSendString = lSendString.."2004="..string.sub(lText,12,21)..":"
			lSendString = lSendString.."2005="..string.sub(lText,23,32)..":"
			lSendString = lSendString.."2006="..string.sub(lText,34,43)..":"
		end
		gLastEKRAN = lText
	end
	
	return lSendString
end

-- Network Functions
function SendConnectMessage()
    io.write("---Log: Connection Request Sent-----", "\n")
	c:send("*C"..gSimID.."\n")
	gAckExpected = 1
    gConnected = false
end

function ProcessInput()
    local lInput = c:receive()
    local lCommand, lCommandArgs, lDevice, lArgument, lLastValue
    
    if lInput then
		--io.write("---Log: Packet on Socket-----", "\n")
        --io.write(lInput.."\n")
        
        lCommand = string.sub(lInput,1,1)
        
        if lCommand == "A" then
        	gAckExpected = gAckExpected - 1
        	if gConnected == false then
        		io.write("---Log: Connected-----", "\n")
        		gConnected = true
        		gAckExpected = 0
        	end
		end
		
		if gConnected then			
			if lCommand == "R" then
    	    	for lArgument, lLastValue in pairs(gIndicators) do
					gIndicators[lArgument] = ""
				end
	       	end
       	
    	   	if lCommand == "Q" then
       			gConnected = false
	       	end	
       	
    	   	if (lCommand == "C" or lCommand == "S") then
    	   		lCommandArgs = StrSplit(lInput,",")
       			lDevice = GetDevice(lCommandArgs[2])
	        	if type(lDevice) == "table" then
    	    		if lCommandArgs[1] == "C" then
        				lDevice:performClickableAction(lCommandArgs[3],lCommandArgs[4])	
	        		elseif lCommandArgs[1] == "S" then
    	    			lDevice:SetCommand(lCommandArgs[3],lCommandArgs[4])
        			end
        		end
	        end
	    end
    end
    
    io.write("---AcksExpected:"..gAckExpected.."-----", "\n")
	io.flush()
	
    if (gConnected == true and gAckExpected > 2) then
		gConnected = false;
		io.write("---Log: Disconnected-----", "\n")
		io.flush()
	end
end

function ProcessOutput()
	if gConnected then
	    local lSendString = ProcessIndicators(gIndicators)
    	lSendString = lSendString..ProcessWeaponSystems()
    	lSendString = lSendString..ProcessEKRAN()
    
	    if lSendString ~= "" then
    	    socket.try(c:send(lSendString.."\n"))
    	    gAckExpected = gAckExpected + 1;
	    end
	else
		SendConnectMessage()
    end  
end

function LuaExportStart()
-- Works once just before mission start.

    -- Open log file for export
    local file = io.open("./Temp/Export.log", "w")

    if file then
        io.output(file)
        io.write("---Log: Start-----", "\n")
        io.flush()
    end
    
    -- 2) Setup udp sockets to talk to touchpal
    package.path  = package.path..";.\\LuaSocket\\?.lua"
    package.cpath = package.cpath..";.\\LuaSocket\\?.dll"
   
    socket = require("socket")
    
    -- Change to correct ip and port (127.0.0.1 = localhost, 9089 = default TouchPal port)
    host = "127.0.0.1"
    port = 9089
    
    c = socket.udp()
    c:setpeername(host, port)
    c:settimeout(.01) -- set the timeout for reading the socket 
    
    SendConnectMessage()
end


function LuaExportBeforeNextFrame()
	ProcessInput()  
end

function LuaExportAfterNextFrame()	
	ProcessOutput()    
end

function LuaExportStop()
-- Works once just after mission stop.
    
	io.write("---Log: End-----", "\n")
    io.flush()    
    io.close()
    
    socket.try(c:send("*Q\n")) -- to close the listener socket
    c:close()
end

function LuaExportActivityNextEvent(t)
	local tNext = t

	return tNext
end